#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include <TFT_eSPI.h>

// Replace with your network credentials
const char* ssid = "Wifi-Name";
const char* password = "Wifi-Password";

// Replace with your API URL and Bearer token
const char* apiUrl = "https://api.todoist.com/rest/v2/tasks";
const char* bearerToken = "Todoist-Api-token";

// OLED display width and height
#define SCREEN_WIDTH 320
#define SCREEN_HEIGHT 240

// Create an instance of the display
TFT_eSPI tft = TFT_eSPI(); // Invoke custom library

void setup() {
  Serial.begin(115200);

  // Initialize the display
  tft.init();
  tft.setRotation(1);
  tft.fillScreen(TFT_BLUE);
  tft.setFreeFont(&FreeSans9pt7b);
  tft.setTextSize(1);
  tft.setTextColor(TFT_WHITE, TFT_BLUE);

  // Connect to Wi-Fi
  Serial.println("Connecting to Wi-Fi...");
  tft.println("Connecting to Wi-Fi...");
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.print(".");
    tft.print(".");
  }
  Serial.println();
  Serial.println("Connected to Wi-Fi");
  tft.println();
  tft.println("Connected to Wi-Fi");

  // Fetch and display the JSON data
  fetchAndDisplayJson();
}

void loop() {
  // Optionally, you can fetch and display data periodically
  //fetchAndDisplayJson();
  //delay(600000); // fetch every 60 seconds
}

void fetchAndDisplayJson() {
  if (WiFi.status() == WL_CONNECTED) {
    HTTPClient http;
    http.begin(apiUrl);
    http.addHeader("Authorization", String("Bearer ") + bearerToken);
    int httpCode = http.GET();

    if (httpCode > 0) {
      if (httpCode == HTTP_CODE_OK) {
        String payload = http.getString();
        Serial.println("Received payload:");
        Serial.println(payload);

        // Parse JSON
        DynamicJsonDocument doc(8192);
        DeserializationError error = deserializeJson(doc, payload);
        if (error) {
          Serial.print(F("deserializeJson() failed: "));
          Serial.println(error.f_str());
          return;
        }

        // Check if the root element is a list
        if (doc.is<JsonArray>()) {
          JsonArray array = doc.as<JsonArray>();

          // Display the JSON array on the TFT
          tft.fillScreen(TFT_BLUE);
          tft.setCursor(0, 0);
          tft.println(" ");
          
          for (JsonObject task : array) {
            if (task["project_id"] == "Project-Id") {
              const char* taskContent = task["content"];
              
              tft.println(String(" ") + taskContent);

              delay(500); // Delay to allow reading, adjust as necessary
            }
          }
        } else {
          Serial.println("Expected a JSON array.");
          tft.fillScreen(TFT_BLUE);
          tft.setCursor(0, 0);
          tft.println("Expected a JSON array.");
        }
      }
    } else {
      Serial.printf("HTTP GET request failed, error: %s\n", http.errorToString(httpCode).c_str());
      tft.fillScreen(TFT_BLUE);
      tft.setCursor(0, 0);
      tft.printf("HTTP GET request failed, error: %s\n", http.errorToString(httpCode).c_str());
    }
    http.end();
  } else {
    Serial.println("Wi-Fi not connected");
    tft.fillScreen(TFT_BLUE);
    tft.setCursor(0, 0);
    tft.println("Wi-Fi not connected");
  }
}
